@@ -16,7 +16,7 @@
 
 # Copyrights (C)
 # for this R-port: 
-#   1999 - 2006, Diethelm Wuertz, GPL
+#   1999 - 2007, Diethelm Wuertz, GPL
 #   Diethelm Wuertz <wuertz@itp.phys.ethz.ch>
 #   info@rmetrics.org
 #   www.rmetrics.org
@@ -38,8 +38,9 @@
 #  fredImport            Downloads market data from St. Louis FED web site
 #  forecastsImport       Downloads monthly data from www.forecasts.org
 # FUNCTION:             IMPORT STATISTICS - EXPERIMENTELL:
-#  keystatsImport        Downloads key statistics from Yahoo's web site  
-#  print.keystats        Print Method for internal function .keystatsImport
+#  keystatsImport       Downloads key statistics from Yahoo's web site 
+#  .keystatsImport      Downloads key statistics from Yahoo's web site  
+#  print.keystats       Print Method for internal function .keystatsImport
 # FUNCTION:             EASY TO USE ROUTINES:
 #  yahooSeries           Easy to use download from Yahoo
 #  .yahooSeries          Utility function  called by 'yahooSeries'
@@ -52,6 +53,7 @@
 
 
 ################################################################################
+# FUNCTION:             DESCRIPTION:
 #  fWEBDATA              Class Representation
 #  show.fWEBDATA         S4 Show Method
 
@@ -103,10 +105,10 @@ setMethod("show", "fWEBDATA", show.fWEBDATA)
 
 
 ################################################################################
+# FUNCTION:             IMPORT DATA FUNCTIONS:
 #  economagicImport      Downloads market data from EconoMagic's web site
 #  yahooImport           Downloads market data from Yahoo's web site 
 #  .yahooImport          ... the old download function 
-#  .keystatsImport       Downloads key statistics from Yahoo's web site  
 #  fredImport            Downloads market data from St. Louis FED web site
 #  forecastsImport       Downloads monthly data from www.forecasts.org
 
@@ -142,8 +144,8 @@ save = FALSE, colname = "VALUE", try = TRUE)
     # FUNCTION:
     
     # Frequency:
-    freq = frequency[1]
-           
+    freq = match.arg(frequency)
+              
     # Download:
     if (try) {
         # First try if the Internet can be accessed:
@@ -263,9 +265,9 @@ swap = 20, try = TRUE)
     #   f     Last Quote ends with Year (yy): as CCYY
     #   r     Aggregation Level
     #   z     Selected Ticker-Symbol [optional]
-
+    
     # Changes:
-    #
+    #   2007-02-18 DW: Update to new %Y-%m-%d format
     
     # FUNCTION:
     
@@ -290,34 +292,21 @@ swap = 20, try = TRUE)
         }
     
         # Download the file:
-        download.file(url = paste(source, query, sep = ""), 
-            destfile = file, method = method)
+        url = paste(source, query, sep = "")
+        download.file(url = url, destfile = file, method = method)
         
-        # Read from file and revert time order:
-        x1 = rev(scan(file, what = "", skip = 1))
+        # Read data and revert:
+        x = X = read.table(file, header = TRUE, sep = ",")
+        n = dim(x)[1]
+        x = x[n:1, ]
         
-        # Extract only date lines including dates:
-        x2 = strsplit(x1[regexpr("-...-..,", x1) > 0], ",")
-        
-        # Create a matrix from the list:
-        x1 = matrix(unlist(x2), byrow = TRUE, nrow = length(x2))
-        
-        # Transfer to numeric data.frame:
-        z = matrix(as.numeric(x1[, -1]), ncol = dim(x1)[2]-1)
-        
-        # Add row (by date) and column (by instrument) names: 
-        # rowNames = as.character(sdate(fjulian(x1[, 1], order = "dmy", 
-        rowNames =  as.character(as.Date(x1[, 1], format = "%d-%b-%y"))
-
-        # DW - add hyphens:
-        # rowNames = paste(
-        #   substring(rowNames, 1, 4), "-",
-        #   substring(rowNames, 5, 6), "-",
-        #   substring(rowNames, 7, 8), sep = "")
-        colNames = scan(file = file, n = dim(x1)[2],  what = "", sep = ",")[-1]
-        dimnames(z) = list(rowNames, colNames)
-            
+        # Result:
+        colnames(x)[1] <- "%Y-%m-%d"
+        rownames(x) = 1:n
+        z = data.frame(x)
+   
         # Save Download ?
+        colNames = colnames(z)[-1]
         if (save) {
             # Header:
             write.table(t(c("%Y-%m-%d", colNames)), file, quote = FALSE, 
@@ -325,13 +314,12 @@ swap = 20, try = TRUE)
             # Data:
             write.table(z, file, quote = FALSE, append = TRUE, 
                 col.names = FALSE, sep = ";") 
+            # Check:
+            # read.table(file, header = TRUE, sep = ";")
         } else {
             unlink(file) 
         } 
         
-        # Result:
-        z = data.frame(DATE = rowNames, z, row.names = NULL)
-        
         # Return Value:
         ans = new("fWEBDATA",     
             call = match.call(),
@@ -376,10 +364,7 @@ frequency = "daily", save = FALSE, sep = ";", try = TRUE)
     #     DGS1      1-Year Treasury Constant Maturity Rate 
     #     DPRIME    Bank Prime Loan Rate
     #      
-
-    # Changes:
-    #
-    
+  
     # FUNCTION:
     
     # Check:
@@ -497,9 +482,6 @@ source = "http://www.forecasts.org/data/data/", save = FALSE, try = TRUE)
     #     EXJPUS        
     #     GS3M
 
-    # Changes:
-    #
-    
     # FUNCTION:
     
     # Download:
@@ -561,9 +543,10 @@ source = "http://www.forecasts.org/data/data/", save = FALSE, try = TRUE)
 
 
 ################################################################################
+# FUNCTION:             IMPORT STATISTICS - EXPERIMENTELL:
 #  keystatsImport       Downloads key statistics from Yahoo's web site 
 #  .keystatsImport      Downloads key statistics from Yahoo's web site  
-#  print.keystats       Print Method for internal function .keystatsImpor
+#  print.keystats       Print Method for internal function .keystatsImport
 
 
 keystatsImport =  
@@ -671,10 +654,7 @@ save = FALSE, try = TRUE)
     
     # Note:
     #   Old Version, no longer used ...
-    
-    # Changes:
-    #
-    
+       
     # FUNCTION:
     
     # Download:
@@ -786,6 +766,7 @@ function(x, ...)
 
 
 ################################################################################
+# FUNCTION:             EASY TO USE ROUTINES:
 #  yahooSeries           Easy to use download from Yahoo
 #  .yahooSeries          Utility function  called by 'yahooSeries'
 
@@ -827,9 +808,6 @@ aggregation = c("d", "w", "m"), returnClass = c("timeSeries", "ts",
     #   yahooSeries(aggregation = "m", nDaysBack = 10*366)
     #   yahooSeries(returnSeries = TRUE)
     
-    # Changes:
-    #
-    
     # FUNCTION:
     
     # Settings:
@@ -911,10 +889,7 @@ returnClass = c("timeSeries", "zoo", "ts"))
     # Examples:
     #   yahooSeries(aggregation = "w")
     #   yahooSeries(aggregation = "m", nDaysBack = 10*366)
-    
-    # Changes:
-    #
-    
+   
     # FUNCTION:
     
     # Automatic Selection of From / To: 
@@ -947,12 +922,29 @@ returnClass = c("timeSeries", "zoo", "ts"))
     # Return Value:
     ans
 }
+
+
+# ------------------------------------------------------------------------------
+
+
+.fredSeries = 
+function(symbol = "DPRIME")
+{   
+    data = fredImport(query = symbol)@data
+    data = as.timeSeries(data)
+    colnames(data) <- symbol
+    data
+}
  
 
 ################################################################################
+# FUNCTION:             EASY TO USE ROUTINES:
+#  yahooSeries           Easy to use download from Yahoo
+#  .yahooSeries          Utility function  called by 'yahooSeries'
+# FUNCTION:             ONLY FOR SPLUS VERSION:
 #  as.Date               S-PLUS: Converts date represenatation
 #  data                  S-PLUS: Loads or lists specified data sets
-#  download.file         S-PLUS: Downloads from Internet using "lynx" or "wget"
+#  download.file         S-PLUS: Downloads using "lynx" or "wget"
 #  strsplit              S-PLUS: Splits character vector into substrings
 
 
@@ -965,9 +957,6 @@ function(x, format = "%d-%m-%y")
     # Description:
     #   Mimics R's as.Date function
     
-    # Changes:
-    #
-    
     # FUNCTION:
     
     # Used by yahooImport ...
@@ -986,9 +975,6 @@ data =
 function(x)
 {   # A function implemented by Diethelm Wuertz
 
-    # Changes:
-    #
-    
     # FUNCTION:
     
     # For S-Plus Compatibility:
@@ -1003,10 +989,7 @@ if (!exists("download.file")) {
 download.file =
 function(url, destfile, method, ...)
 {   # A function implemented by Diethelm Wuertz
-    
-    # Changes:
-    #
-    
+  
     # FUNCTION:
     
     # Download:
@@ -1030,10 +1013,7 @@ if (!exists("strsplit")) {
 strsplit = 
 function(x, split = " ") 
 {   # A function implemented by Diethelm Wuertz
-
-    # Changes:
-    #
-    
+  
     # FUNCTION:
     
     # For S-Plus Compatibility:
@@ -1044,6 +1024,5 @@ function(x, split = " ")
 }}
  
 
-
 ################################################################################
 
